/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 package net.micode.notes.ui;

 import java.util.Calendar;
 
 import net.micode.notes.R;
 import net.micode.notes.ui.DateTimePicker;
 import net.micode.notes.ui.DateTimePicker.OnDateTimeChangedListener;
 
 import android.app.AlertDialog;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.DialogInterface.OnClickListener;
 import android.text.format.DateFormat;
 import android.text.format.DateUtils;
 
 /**
  * DateTimePickerDialog 类继承自 AlertDialog，用于显示一个日期时间选择对话框。
  * 该对话框包含一个 DateTimePicker 组件，允许用户选择日期和时间。
  * 当用户选择日期和时间并点击确定按钮时，会触发相应的回调函数。
  */
 public class DateTimePickerDialog extends AlertDialog implements OnClickListener {
 
     // 用于存储用户选择的日期和时间
     private Calendar mDate = Calendar.getInstance();
     // 标记是否使用 24 小时制显示时间
     private boolean mIs24HourView;
     // 日期时间选择确定的回调接口
     private OnDateTimeSetListener mOnDateTimeSetListener;
     // 自定义的日期时间选择器组件
     private DateTimePicker mDateTimePicker;
 
     /**
      * 日期时间选择确定的回调接口，当用户点击确定按钮时会调用该接口的方法。
      */
     public interface OnDateTimeSetListener {
         /**
          * 当用户点击确定按钮时调用该方法，传递对话框实例和选择的日期时间（以毫秒为单位）。
          * @param dialog 日期时间选择对话框实例
          * @param date 选择的日期时间（以毫秒为单位）
          */
         void OnDateTimeSet(AlertDialog dialog, long date);
     }
 
     /**
      * 构造函数，初始化日期时间选择对话框。
      * @param context 上下文对象
      * @param date 初始显示的日期时间（以毫秒为单位）
      */
     public DateTimePickerDialog(Context context, long date) {
         super(context);
         // 创建 DateTimePicker 实例
         mDateTimePicker = new DateTimePicker(context);
         // 将 DateTimePicker 设置为对话框的视图
         setView(mDateTimePicker);
         // 为 DateTimePicker 设置日期时间改变的监听器
         mDateTimePicker.setOnDateTimeChangedListener(new OnDateTimeChangedListener() {
             @Override
             public void onDateTimeChanged(DateTimePicker view, int year, int month,
                     int dayOfMonth, int hourOfDay, int minute) {
                 // 更新 Calendar 对象的日期和时间
                 mDate.set(Calendar.YEAR, year);
                 mDate.set(Calendar.MONTH, month);
                 mDate.set(Calendar.DAY_OF_MONTH, dayOfMonth);
                 mDate.set(Calendar.HOUR_OF_DAY, hourOfDay);
                 mDate.set(Calendar.MINUTE, minute);
                 // 更新对话框的标题显示选择的日期时间
                 updateTitle(mDate.getTimeInMillis());
             }
         });
         // 设置 Calendar 对象的时间为传入的日期时间
         mDate.setTimeInMillis(date);
         // 将秒数设置为 0
         mDate.set(Calendar.SECOND, 0);
         // 设置 DateTimePicker 的当前日期时间
         mDateTimePicker.setCurrentDate(mDate.getTimeInMillis());
         // 设置对话框的确定按钮及其点击监听器
         setButton(context.getString(R.string.datetime_dialog_ok), this);
         // 设置对话框的取消按钮
         setButton2(context.getString(R.string.datetime_dialog_cancel), (OnClickListener)null);
         // 根据系统设置确定是否使用 24 小时制
         set24HourView(DateFormat.is24HourFormat(this.getContext()));
         // 更新对话框的标题显示初始日期时间
         updateTitle(mDate.getTimeInMillis());
     }
 
     /**
      * 设置是否使用 24 小时制显示时间。
      * @param is24HourView true 表示使用 24 小时制，false 表示使用 12 小时制
      */
     public void set24HourView(boolean is24HourView) {
         mIs24HourView = is24HourView;
     }
 
     /**
      * 设置日期时间选择确定的回调接口。
      * @param callBack 实现 OnDateTimeSetListener 接口的回调对象
      */
     public void setOnDateTimeSetListener(OnDateTimeSetListener callBack) {
         mOnDateTimeSetListener = callBack;
     }
 
     /**
      * 更新对话框的标题，显示当前选择的日期时间。
      * @param date 当前选择的日期时间（以毫秒为单位）
      */
     private void updateTitle(long date) {
         // 设置日期时间显示的标志
         int flag =
             DateUtils.FORMAT_SHOW_YEAR |
             DateUtils.FORMAT_SHOW_DATE |
             DateUtils.FORMAT_SHOW_TIME;
         // 根据是否使用 24 小时制添加相应的标志
         flag |= mIs24HourView ? DateUtils.FORMAT_24HOUR : DateUtils.FORMAT_24HOUR;
         // 设置对话框的标题为格式化后的日期时间
         setTitle(DateUtils.formatDateTime(this.getContext(), date, flag));
     }
 
     /**
      * 处理对话框按钮的点击事件。
      * @param arg0 对话框接口对象
      * @param arg1 按钮的索引
      */
     @Override
     public void onClick(DialogInterface arg0, int arg1) {
         // 如果设置了回调接口，则调用回调方法
         if (mOnDateTimeSetListener != null) {
             mOnDateTimeSetListener.OnDateTimeSet(this, mDate.getTimeInMillis());
         }
     }
 }