/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 package net.micode.notes.ui;

 import android.content.Context;
 import android.view.Menu;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.View.OnClickListener;
 import android.widget.Button;
 import android.widget.PopupMenu;
 import android.widget.PopupMenu.OnMenuItemClickListener;
 
 import net.micode.notes.R;
 
 /**
  * DropdownMenu 类用于创建一个下拉菜单，它将一个按钮与一个弹出菜单关联起来。
  * 当点击按钮时，会显示弹出菜单。
  */
 public class DropdownMenu {
     // 用于触发弹出菜单的按钮
     private Button mButton;
     // 弹出菜单对象
     private PopupMenu mPopupMenu;
     // 弹出菜单的菜单对象
     private Menu mMenu;
 
     /**
      * 构造函数，用于初始化下拉菜单。
      *
      * @param context 上下文对象，用于创建弹出菜单
      * @param button  触发弹出菜单的按钮
      * @param menuId  菜单资源的 ID，用于指定弹出菜单的内容
      */
     public DropdownMenu(Context context, Button button, int menuId) {
         // 保存传入的按钮对象
         mButton = button;
         // 设置按钮的背景资源为下拉图标
         mButton.setBackgroundResource(R.drawable.dropdown_icon);
         // 创建一个弹出菜单，将其关联到按钮上
         mPopupMenu = new PopupMenu(context, mButton);
         // 获取弹出菜单的菜单对象
         mMenu = mPopupMenu.getMenu();
         // 从指定的菜单资源 ID 中加载菜单内容到菜单对象中
         mPopupMenu.getMenuInflater().inflate(menuId, mMenu);
         // 为按钮设置点击监听器，当按钮被点击时显示弹出菜单
         mButton.setOnClickListener(new OnClickListener() {
             public void onClick(View v) {
                 mPopupMenu.show();
             }
         });
     }
 
     /**
      * 设置下拉菜单中菜单项的点击监听器。
      *
      * @param listener 菜单项点击监听器对象
      */
     public void setOnDropdownMenuItemClickListener(OnMenuItemClickListener listener) {
         // 检查弹出菜单是否为空
         if (mPopupMenu != null) {
             // 设置弹出菜单的菜单项点击监听器
             mPopupMenu.setOnMenuItemClickListener(listener);
         }
     }
 
     /**
      * 根据菜单项的 ID 查找对应的菜单项。
      *
      * @param id 菜单项的 ID
      * @return 找到的菜单项对象，如果未找到则返回 null
      */
     public MenuItem findItem(int id) {
         // 在菜单对象中查找指定 ID 的菜单项
         return mMenu.findItem(id);
     }
 
     /**
      * 设置按钮的标题文本。
      *
      * @param title 要设置的标题文本
      */
     public void setTitle(CharSequence title) {
         // 设置按钮的文本为传入的标题文本
         mButton.setText(title);
     }
 }